#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library
import json

# import interfaces
from hypatia.interfaces import ICatalog
from pyams_form.interfaces.form import IFormLayer
from pyams_i18n.interfaces import II18n
from pyams_sequence.interfaces import ISequentialIntIds, ISequentialIdInfo
from pyams_sequence.schema import IInternalReferenceField, IInternalReferencesListField
from pyams_sequence.widget.interfaces import IInternalReferenceWidget, IInternalReferencesListWidget

try:
    from pyams_workflow.interfaces import IWorkflowVersions, IWorkflowVersion, IWorkflowManagedContent
except ImportError:
    handle_workflow = False
else:
    handle_workflow = True

from z3c.form.interfaces import IFieldWidget, IDataConverter

# import packages
from hypatia.catalog import CatalogQuery
from hypatia.query import Eq, Any
from pyams_catalog.query import CatalogResultSet
from pyams_form.widget import widgettemplate_config
from pyams_sequence.reference import get_last_version
from pyams_utils.adapter import adapter_config
from pyams_utils.registry import get_utility
from z3c.form.browser.widget import HTMLInputWidget
from z3c.form.converter import BaseDataConverter
from z3c.form.widget import Widget, FieldWidget
from zope.interface import implementer_only

from pyams_sequence import _


#
# Internal reference widget
#

@widgettemplate_config(mode='input', template='templates/reference-input.pt', layer=IFormLayer)
@widgettemplate_config(mode='display', template='templates/reference-display.pt', layer=IFormLayer)
@implementer_only(IInternalReferenceWidget)
class InternalReferenceWidget(HTMLInputWidget, Widget):
    """Internal reference widget"""

    content_type = None

    @property
    def query_params(self):
        return json.dumps({'content_type': self.content_type or self.field.content_type})

    @property
    def values_map(self):
        catalog = get_utility(ICatalog)
        params = Eq(catalog['oid'], self.value)
        results = list(map(get_last_version, CatalogResultSet(CatalogQuery(catalog).query(params))))
        if results:
            sequence = get_utility(ISequentialIntIds)
            return json.dumps({self.value: '{0} ({1})'.format(II18n(results[0]).query_attribute('title',
                                                                                                request=self.request),
                                                              sequence.get_short_oid(ISequentialIdInfo(results[0]).oid))})
        else:
            translate = self.request.localizer.translate
            return json.dumps({self.value: translate(_("missing reference: {0}")).format(self.value)})


@adapter_config(context=(IInternalReferenceField, IFormLayer), provides=IFieldWidget)
def InternalReferenceFieldWidget(field, request):
    """Internal reference field widget factory"""
    return FieldWidget(field, InternalReferenceWidget(request))


#
# Internal references list widget
#

@adapter_config(context=(IInternalReferencesListField, IInternalReferencesListWidget), provides=IDataConverter)
class InternalReferencesListDataConverter(BaseDataConverter):
    """Internal references list data converter"""

    def toWidgetValue(self, value):
        if not value:
            return ''
        return ','.join(value)

    def toFieldValue(self, value):
        if not value:
            return []
        return value.split(',')


@widgettemplate_config(mode='input', template='templates/references-list-input.pt', layer=IFormLayer)
@widgettemplate_config(mode='display', template='templates/references-list-display.pt', layer=IFormLayer)
@implementer_only(IInternalReferencesListWidget)
class InternalReferencesListWidget(HTMLInputWidget, Widget):
    """Internal references list widget"""

    content_type = None

    @property
    def query_params(self):
        return json.dumps({'content_type': self.content_type or self.field.content_type})

    @property
    def values_map(self):
        catalog = get_utility(ICatalog)
        sequence = get_utility(ISequentialIntIds)
        results = {}
        values = self.value
        if values:
            if isinstance(values, str):
                values = values.split(',')
            params = Any(catalog['oid'], values)
            for item in map(get_last_version, CatalogResultSet(CatalogQuery(catalog).query(params))):
                oid_info = ISequentialIdInfo(item)
                results.update({oid_info.hex_oid: '{0} ({1})'.format(II18n(item).query_attribute('title',
                                                                                                 request=self.request),
                                                                     sequence.get_short_oid(oid_info.oid))})
        return json.dumps(results)


@adapter_config(context=(IInternalReferencesListField, IFormLayer), provides=IFieldWidget)
def InternalReferencesListFieldWidget(field, request):
    """Internal references list field widget factory"""
    return FieldWidget(field, InternalReferencesListWidget(request))
